@@ -34,12 +34,12 @@ try:
 except ImportError:
     print("Error: paramiko is a required module. Please install it:")
     print("  $ sudo easy_install paramiko")
-    exit(1)
+    sys.exit(1)
 
-__version__ = '0.0.4'
+__version__ = '0.0.6'
 __author__ = 'Christian Vest Hansen'
 __author_email__ = 'karmazilla@gmail.com'
-__url__ = 'https://savannah.nongnu.org/projects/fab/'
+__url__ = 'http://www.nongnu.org/fab/'
 __license__ = 'GPL-2'
 __greeter__ = '''\
    Fabric v. %(fab_version)s, Copyright (C) 2008 %(fab_author)s.
@@ -49,17 +49,18 @@ __greeter__ = '''\
 '''
 
 ENV = {
-    'fab_version':__version__,
-    'fab_author':__author__,
-    'fab_mode':'rolling',
-    'fab_port':22,
-    'fab_user':None,
-    'fab_pkey':None,
-    'fab_key_filename':None,
-    'fab_new_host_key':'accept',
-    'fab_shell':'/bin/bash -l -c "%s"',
-    'fab_timestamp':datetime.datetime.utcnow().strftime('%F_%H-%M-%S'),
-    'fab_print_real_sudo':False,
+    'fab_version': __version__,
+    'fab_author': __author__,
+    'fab_mode': 'rolling',
+    'fab_port': 22,
+    'fab_user': os.getlogin(),
+    'fab_password': None,
+    'fab_pkey': None,
+    'fab_key_filename': None,
+    'fab_new_host_key': 'accept',
+    'fab_shell': '/bin/bash -l -c "%s"',
+    'fab_timestamp': datetime.datetime.utcnow().strftime('%F_%H-%M-%S'),
+    'fab_print_real_sudo': False,
 }
 
 CONNECTIONS = []
@@ -68,6 +69,19 @@ OPERATIONS = {}
 STRATEGIES = {}
 _LAZY_FORMAT_SUBSTITUTER = re.compile(r'\$\((?P<var>\w+?)\)')
 
+#
+# Compatibility fixes
+#
+if hasattr(str, 'partition'):
+    partition = str.partition
+else:
+    def partition(txt, sep):
+        idx = txt.find(sep)
+        if idx == -1:
+            return txt, '', ''
+        else:
+            return (txt[:idx], sep, txt[idx + len(sep):])
+
 #
 # Helper decorators:
 #
@@ -87,10 +101,10 @@ operation = new_registering_decorator(OPERATIONS)
 strategy = new_registering_decorator(STRATEGIES)
 
 def run_per_host(op_fn):
-    def wrapper(*args, **kvargs):
+    def wrapper(*args, **kwargs):
         if not CONNECTIONS:
             _connect()
-        _on_hosts_do(op_fn, *args, **kvargs)
+        _on_hosts_do(op_fn, *args, **kwargs)
     wrapper.__doc__ = op_fn.__doc__
     wrapper.__name__ = op_fn.__name__
     return wrapper
@@ -100,14 +114,15 @@ def run_per_host(op_fn):
 #
 @operation
 def set(**variables):
-    """Set a number of Fabric environment variables.
+    """
+    Set a number of Fabric environment variables.
     
-    Set takes a number of keyword arguments, and defines or updates the
-    variables that corrosponds to each keyword with the respective value.
+    set() takes a number of keyword arguments, and defines or updates the
+    variables that correspond to each keyword with the respective value.
     
     The values can be of any type, but strings are used for most variables.
     If the value is a string and contain any eager variable references, such as
-    %(fab_user)s, then these will be expanded to their corrosponding value.
+    %(fab_user)s, then these will be expanded to their corresponding value.
     Lazy references, those beginning with a $ rather than a %, will not be
     expanded.
     
@@ -123,20 +138,39 @@ def set(**variables):
 
 @operation
 def get(name, otherwise=None):
-    """Get the value of a given Fabric environment variable.
+    """
+    Get the value of a given Fabric environment variable.
     
     If the variable isn't found, then this operation returns the
     value of the 'otherwise' parameter, which is None unless set.
     
     """
-    return name in ENV and ENV[name] or otherwise
+    return ENV.get(name, otherwise)
 
 @operation
-def require(var, **kvargs):
-    """Make sure that a certain environmet variable is available.
+def getAny(*names):
+    """
+    Given a list of variable names as parameters, get the value of the first
+    of these variables that is actually defined (and does not resolve to
+    boolean False), or None.
+    
+    Example:
+        getAny('hostname', 'ipv4', 'ipv6', 'ip', 'address')
+    
+    """
+    for name in names:
+        value = ENV.get(name)
+        if value:
+            return value
+    # Implicit return value of None here if no names found.
+
+@operation
+def require(var, **kwargs):
+    """
+    Make sure that a certain environment variable is available.
     
     The 'var' parameter is a string that names the variable to check for.
-    Two other optional kvargs are supported:
+    Two other optional kwargs are supported:
         * 'used_for' is a string that gets injected into, and then printed, as
           something like this string: "This variable is used for %s".
         * 'provided_by' is a list of strings that name commands which the user
@@ -158,19 +192,63 @@ def require(var, **kvargs):
         ("The '%(fab_cur_command)s' command requires a '" + var
         + "' variable.") % ENV
     )
-    if 'used_for' in kvargs:
-        print("This variable is used for %s" % _lazy_format(kvargs['used_for']))
-    if 'provided_by' in kvargs:
+    if 'used_for' in kwargs:
+        print("This variable is used for %s" % _lazy_format(
+            kwargs['used_for']))
+    if 'provided_by' in kwargs:
         print("Get the variable by running one of these commands:")
         to_s = lambda obj: getattr(obj, '__name__', str(obj))
-        provided_by = [to_s(obj) for obj in kvargs['provided_by']]
+        provided_by = [to_s(obj) for obj in kwargs['provided_by']]
         print('\t' + ('\n\t'.join(provided_by)))
-    exit(1)
+    sys.exit(1)
+
+@operation
+def prompt(varname, msg, validate=None, default=None):
+    """
+    Display a prompt to the user and store the input in the given variable.
+    If the variable already exists, then it is not prompted for again.
+    
+    The 'validate' parameter is a callable that raises an exception on invalid
+    inputs and returns the input for storage in ENV.
+    It may process the input and convert it to a different type, as in the
+    second example below.
+    
+    Example:
+        # Simplest form:
+        prompt('environment', 'Please specify target environment')
+        
+        # With default:
+        prompt('dish', 'Specify favorite dish', default='spam & eggs')
+        
+        # With validation, i.e. require integer input:
+        prompt('nice', 'Please specify process nice level', validate=int)
+    
+    """
+    if varname in ENV and ENV[varname] is not None:
+        return
+    
+    if callable(default):
+        default = default()
+    
+    try:
+        default_str = default and (" [%s]" % str(default).strip()) or ""
+        prompt_msg = _lazy_format("%s%s: " % (msg.strip(), default_str))
+        value = raw_input(prompt_msg)
+        if not value:
+            value = default
+        
+        if callable(validate):
+            value = validate(value)
+        
+        set(**{varname: value})
+    except EOFError:
+        return
 
 @operation
 @run_per_host
-def put(host, client, env, localpath, remotepath, **kvargs):
-    """Upload a file to the current hosts.
+def put(host, client, env, localpath, remotepath, **kwargs):
+    """
+    Upload a file to the current hosts.
     
     The 'localpath' parameter is the relative or absolute path to the file on
     your localhost that you wish to upload to the fab_hosts.
@@ -198,8 +276,9 @@ def put(host, client, env, localpath, remotepath, **kvargs):
 
 @operation
 @run_per_host
-def download(host, client, env, remotepath, localpath, **kvargs):
-    """Download a file from the remote hosts.
+def download(host, client, env, remotepath, localpath, **kwargs):
+    """
+    Download a file from the remote hosts.
     
     The 'remotepath' parameter is the relative or absolute path to the files
     to download from the fab_hosts. The 'localpath' parameter will be suffixed
@@ -212,7 +291,7 @@ def download(host, client, env, remotepath, localpath, **kvargs):
         * abort - terminate fabric on failure
     
     Example:
-        set(fab_hosts=['node1.cluster.com','node2.cluster.com'])
+        set(fab_hosts=['node1.cluster.com', 'node2.cluster.com'])
         download('/var/log/server.log', 'server.log')
     
     The above code will produce two files on your local system, called
@@ -229,10 +308,11 @@ def download(host, client, env, remotepath, localpath, **kvargs):
 
 @operation
 @run_per_host
-def run(host, client, env, cmd, **kvargs):
-    """Run a shell command on the current fab_hosts.
+def run(host, client, env, cmd, **kwargs):
+    """
+    Run a shell command on the current fab_hosts.
     
-    The provided command is executed with the permisions of fab_user, and the
+    The provided command is executed with the permissions of fab_user, and the
     exact execution environ is determined by the fab_shell variable.
     
     May take an additional 'fail' keyword argument with one of these values:
@@ -245,7 +325,7 @@ def run(host, client, env, cmd, **kvargs):
     
     """
     cmd = env['fab_shell'] % _lazy_format(cmd, env).replace('"', '\\"')
-    if not _confirm_proceed('run', host, kvargs):
+    if not _confirm_proceed('run', host, kwargs):
         return False # TODO: should we return False in fail??
     print("[%s] run: %s" % (host, cmd))
     stdin, stdout, stderr = client.exec_command(cmd)
@@ -257,10 +337,11 @@ def run(host, client, env, cmd, **kvargs):
 
 @operation
 @run_per_host
-def sudo(host, client, env, cmd, **kvargs):
-    """Run a sudo (root privileged) command on the current hosts.
+def sudo(host, client, env, cmd, **kwargs):
+    """
+    Run a sudo (root privileged) command on the current hosts.
     
-    The provided command is executed with root permisions, provided that
+    The provided command is executed with root permissions, provided that
     fab_user is in the sudoers file in the remote host. The exact execution
     environ is determined by the fab_shell variable - the 'sudo' part is
     injected into this variable.
@@ -275,15 +356,18 @@ def sudo(host, client, env, cmd, **kvargs):
     
     """
     cmd = _lazy_format(cmd, env)
-    real_cmd = env['fab_shell'] % ("sudo -S " + cmd.replace('"', '\\"'))
+    passwd = get('fab_password')
+    sudo_cmd = passwd and "sudo -S " or "sudo "
+    real_cmd = env['fab_shell'] % (sudo_cmd + cmd.replace('"', '\\"'))
     cmd = env['fab_print_real_sudo'] and real_cmd or cmd
-    if not _confirm_proceed('sudo', host, kvargs):
+    if not _confirm_proceed('sudo', host, kwargs):
         return False # TODO: should we return False in fail??
     print("[%s] sudo: %s" % (host, cmd))
     stdin, stdout, stderr = client.exec_command(real_cmd)
-    stdin.write(env['fab_password'])
-    stdin.write('\n')
-    stdin.flush()
+    if passwd:
+        stdin.write(env['fab_password'])
+        stdin.write('\n')
+        stdin.flush()
     out_th = _start_outputter("[%s] out" % host, stdout)
     err_th = _start_outputter("[%s] err" % host, stderr)
     out_th.join()
@@ -291,8 +375,9 @@ def sudo(host, client, env, cmd, **kvargs):
     return True
 
 @operation
-def local(cmd, **kvargs):
-    """Run a command locally.
+def local(cmd, **kwargs):
+    """
+    Run a command locally.
     
     This operation is essentially 'os.system()' except that variables are
     expanded prior to running.
@@ -303,23 +388,24 @@ def local(cmd, **kvargs):
         * abort - terminate fabric on failure
     
     Example:
-        local("make clean dist")
+        local("make clean dist", fail='abort')
     
     """
     final_cmd = _lazy_format(cmd)
     print("[localhost] run: " + final_cmd)
     retcode = subprocess.call(final_cmd, shell=True)
     if retcode != 0:
-        failcode = _get_failcode(kvargs)
+        failcode = _get_failcode(kwargs)
         if failcode > 1:
             print("Warning: failed to execute command:")
             print("\t" + final_cmd)
         if failcode > 2:
-            exit(1)
+            sys.exit(1)
 
 @operation
-def local_per_host(cmd, **kvargs):
-    """Run a command locally, for every defined host.
+def local_per_host(cmd, **kwargs):
+    """
+    Run a command locally, for every defined host.
     
     Like the local() operation, this is pretty similar to 'os.system()', but
     with this operation, the command is executed (and have its variables
@@ -334,11 +420,12 @@ def local_per_host(cmd, **kvargs):
     _check_fab_hosts()
     for host in ENV['fab_hosts']:
         ENV['fab_host'] = host
-        local(cmd, **kvargs)
+        local(cmd, **kwargs)
 
 @operation
-def load(filename, **kvargs):
-    """Load up the given fabfile.
+def load(filename, **kwargs):
+    """
+    Load up the given fabfile.
     
     This loads the fabfile specified by the 'filename' parameter into fabric
     and make its commands and other functions available in the scope of the 
@@ -353,7 +440,7 @@ def load(filename, **kvargs):
         load("conf/production-settings.py")
     
     """
-    failcode = _get_failcode(kvargs)
+    failcode = _get_failcode(kwargs)
     if os.path.exists(filename):
         execfile(filename)
         for name, obj in locals().items():
@@ -365,11 +452,12 @@ def load(filename, **kvargs):
         print("Warning: Cannot load file '%s'." % filename)
         print("No such file in your current directory.")
         if failcode > 2:
-            exit(1)
+            sys.exit(1)
 
 @operation
-def upload_project(**kvargs):
-    """Uploads the current project directory to the connected hosts.
+def upload_project(**kwargs):
+    """
+    Uploads the current project directory to the connected hosts.
     
     This is a higher-level convenience operation that basically 'tar' up the
     directory that contains your fabfile (presumably it is your project
@@ -385,18 +473,19 @@ def upload_project(**kvargs):
     """
     tar_file = "/tmp/fab.%(fab_timestamp)s.tar" % ENV
     cwd_name = os.getcwd().split(os.sep)[-1]
-    local("tar -czf %s ." % tar_file, **kvargs)
-    put(tar_file, cwd_name + ".tar.gz", **kvargs)
-    local("rm -f " + tar_file, **kvargs)
-    run("tar -xzf " + cwd_name, **kvargs)
-    run("rm -f " + cwd_name + ".tar.gz", **kvargs)
+    local("tar -czf %s ." % tar_file, **kwargs)
+    put(tar_file, cwd_name + ".tar.gz", **kwargs)
+    local("rm -f " + tar_file, **kwargs)
+    run("tar -xzf " + cwd_name, **kwargs)
+    run("rm -f " + cwd_name + ".tar.gz", **kwargs)
 
 #
 # Standard Fabric commands:
 #
 @command("help")
-def _help(**kvargs):
-    """Display Fabric usage help, or help for a given command.
+def _help(**kwargs):
+    """
+    Display Fabric usage help, or help for a given command.
     
     You can provide help with a parameter and get more detailed help for a
     specific command. For instance, to learn more about the list command, you
@@ -412,20 +501,21 @@ def _help(**kvargs):
     and 'strategy' parameters: 'fab help:strg=rolling'.
     
     """
-    if kvargs:
-        for k, v in kvargs.items():
+    if kwargs:
+        for k, v in kwargs.items():
             if k in COMMANDS:
                 _print_help_for_in(k, COMMANDS)
             elif k in OPERATIONS:
                 _print_help_for_in(k, OPERATIONS)
             elif k in ['op', 'operation']:
-                _print_help_for_in(kvargs[k], OPERATIONS)
+                _print_help_for_in(kwargs[k], OPERATIONS)
             elif k in ['strg', 'strategy']:
-                _print_help_for_in(kvargs[k], STRATEGIES)
+                _print_help_for_in(kwargs[k], STRATEGIES)
             else:
                 _print_help_for(k, None)
     else:
-        print("""Fabric is a simple pythonic remote deployment tool.
+        print("""
+    Fabric is a simple pythonic remote deployment tool.
     
     Type 'fab list' to get a list of available commands.
     Type 'fab help:help' to get more information on how to use the built in
@@ -434,18 +524,19 @@ def _help(**kvargs):
     """)
 
 @command("list")
-def _list_commands(**kvargs):
-    """Display a list of commands with descriptions.
+def _list_commands(**kwargs):
+    """
+    Display a list of commands with descriptions.
     
     By default, the list command prints a list of available commands, with a
     short description (if one is available). However, the list command can also
-    print a list of available operaions if you provide it with the 'ops' or
+    print a list of available operations if you provide it with the 'ops' or
     'operations' parameters, or it can print strategies with the 'strgs' and
     'strategies' parameters.
     
     """
-    if kvargs:
-        for k, v in kvargs.items():
+    if kwargs:
+        for k, v in kwargs.items():
             if k in ['cmds', 'commands']:
                 print("Available commands are:")
                 _list_objs(COMMANDS)
@@ -463,7 +554,7 @@ def _list_commands(**kvargs):
                     'ops', 'operations',
                     'strgs', 'strategies',
                 ])))
-                exit(1)
+                sys.exit(1)
     else:
         print("Available commands are:")
         _list_objs(COMMANDS)
@@ -696,17 +787,18 @@ PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGES.""")
 
 @command("set")
-def _set(**kvargs):
-    """Set a Fabric variable.
+def _set(**kwargs):
+    """
+    Set a Fabric variable.
     
     Example:
         $fab set:fab_user=billy,other_var=other_value
     """
-    for k, v in kvargs.items():
+    for k, v in kwargs.items():
         ENV[k] = (v % ENV)
 
 @command("shell")
-def _shell(**kvargs):
+def _shell(**kwargs):
     "Start an interactive shell connection to the specified hosts."
     def lines():
         try:
@@ -727,8 +819,9 @@ def _shell(**kvargs):
 # Standard strategies:
 #
 @strategy("fanout")
-def _fanout_strategy(fn, *args, **kvargs):
-    """A strategy that executes on all hosts in parallel.
+def _fanout_strategy(fn, *args, **kwargs):
+    """
+    A strategy that executes on all hosts in parallel.
     
     THIS STRATEGY IS CURRENTLY BROKEN!
     
@@ -739,7 +832,7 @@ def _fanout_strategy(fn, *args, **kvargs):
         env = dict(ENV)
         env['fab_host'] = host
         def functor():
-            success = success and fn(host, client, env, *args, **kvargs)
+            success = success and fn(host, client, env, *args, **kwargs)
         thread = threading.Thread(None, functor)
         thread.setDaemon(True)
         thread.start()
@@ -749,12 +842,12 @@ def _fanout_strategy(fn, *args, **kvargs):
     return success
 
 @strategy("rolling")
-def _rolling_strategy(fn, *args, **kvargs):
+def _rolling_strategy(fn, *args, **kwargs):
     """One-at-a-time fail-fast strategy."""
     for host, client in CONNECTIONS:
         env = dict(ENV)
         env['fab_host'] = host
-        if not fn(host, client, env, *args, **kvargs):
+        if not fn(host, client, env, *args, **kwargs):
             return False
     return True
 
@@ -778,9 +871,9 @@ def _print_help_for_in(name, dictionary):
         _print_help_for(name, None)
 
 def _list_objs(objs):
-    max_name_len = reduce(lambda a,b: max(a, len(b)), objs.keys(), 0)
+    max_name_len = reduce(lambda a, b: max(a, len(b)), objs.keys(), 0)
     cmds = objs.items()
-    cmds.sort(lambda x,y: cmp(x[0], y[0]))
+    cmds.sort(lambda x, y: cmp(x[0], y[0]))
     for name, fn in cmds:
         print '  ', name.ljust(max_name_len),
         if fn.__doc__:
@@ -794,38 +887,42 @@ def _check_fab_hosts():
         print("Fabric requires a fab_hosts variable.")
         print("Please set it in your fabfile.")
         print("Example: set(fab_hosts=['node1.com', 'node2.com'])")
-        exit(1)
+        sys.exit(1)
 
 def _connect():
     "Populate CONNECTIONS with (hostname, client) tuples as per fab_hosts."
     _check_fab_hosts()
-    signal.signal(signal.SIGINT, lambda: _disconnect() and exit(0))
-    if 'fab_password' not in ENV:
+    signal.signal(signal.SIGINT, lambda: _disconnect() and sys.exit(0))
+    if not get('fab_connected'):
         print(_lazy_format("Logging into the following hosts as $(fab_user):"))
         print(_indent('\n'.join(ENV['fab_hosts'])))
-        ENV['fab_password'] = getpass.getpass()
-    else:
-        print("Warning: Putting your password in a fabfile is a bad idea.")
     def_port = ENV['fab_port']
     username = ENV['fab_user']
     password = ENV['fab_password']
     pkey = ENV['fab_pkey']
     key_filename = ENV['fab_key_filename']
     for host in ENV['fab_hosts']:
-        host, _, port = host.partition(':')
+        host, _, port = partition(host, ':')
         portnr = int(port or def_port)
         client = ssh.SSHClient()
         client.load_system_host_keys()
         if 'fab_new_host_key' in ENV and ENV['fab_new_host_key'] == 'accept':
             client.set_missing_host_key_policy(ssh.AutoAddPolicy())
-        client.connect(
-            host, portnr, username, password, pkey, key_filename
-        )
+        try:
+            client.connect(
+                host, portnr, username, password, pkey, key_filename
+            )
+        except ssh.AuthenticationException:
+            password = ENV['fab_password'] = getpass.getpass()
+            client.connect(
+                host, portnr, username, password, pkey, key_filename
+            )
         CONNECTIONS.append((host, client))
     if not CONNECTIONS:
         print("The fab_hosts list was empty.")
         print("Please specify some hosts to connect to.")
-        exit(1)
+        sys.exit(1)
+    set(fab_connected=True)
 
 def _disconnect():
     "Disconnect all clients."
@@ -835,9 +932,10 @@ def _disconnect():
     CONNECTIONS = []
 
 def _lazy_format(string, env=ENV):
-    "Do recursive string substitution of ENV vars - both lazy and earger."
+    "Do recursive string substitution of ENV vars - both lazy and eager."
     if string is None:
         return None
+    env = dict([(k, str(v)) for k, v in env.items()])
     def replacer_fn(match):
         var = match.group('var')
         if var in env:
@@ -846,32 +944,33 @@ def _lazy_format(string, env=ENV):
             return match.group(0)
     return re.sub(_LAZY_FORMAT_SUBSTITUTER, replacer_fn, string % env)
 
-def _on_hosts_do(fn, *args, **kvargs):
-    """Invoke the given function with hostname and client parameters in
+def _on_hosts_do(fn, *args, **kwargs):
+    """
+    Invoke the given function with hostname and client parameters in
     accord with the current fac_mode strategy.
     
     fn should be a callable taking these parameters:
         hostname : str
         client : paramiko.SSHClient
         *args
-        **kvargs
+        **kwargs
     
     """
     strategy = ENV['fab_mode']
     if strategy in STRATEGIES:
         strategy_fn = STRATEGIES[strategy]
-        successful = strategy_fn(fn, *args, **kvargs)
+        successful = strategy_fn(fn, *args, **kwargs)
         if not successful:
             print("Operation failed: " + fn.__name__)
-            exit(1)
+            sys.exit(1)
     else:
         print("Unsupported fab_mode: %s" % strategy)
-        print("Supported modes are: fanout, rolling")
-        exit(1)
+        print("Supported modes are: %s" % (', '.join(STRATEGIES.keys())))
+        sys.exit(1)
 
-def _confirm_proceed(exec_type, host, kvargs):
-    if 'confirm' in kvargs:
-        infotuple = (exec_type, host, _lazy_format(kvargs['confirm']))
+def _confirm_proceed(exec_type, host, kwargs):
+    if 'confirm' in kwargs:
+        infotuple = (exec_type, host, _lazy_format(kwargs['confirm']))
         question = "Confirm %s for host %s: %s [yN] " % infotuple
         answer = raw_input(question)
         return answer in 'yY'
@@ -901,7 +1000,7 @@ def _start_outputter(prefix, channel):
 
 def _pick_fabfile():
     "Figure out what the fabfile is called."
-    guesses = ['fabfile','Fabfile', 'fabfile.py', 'Fabfile.py']
+    guesses = ['fabfile', 'Fabfile', 'fabfile.py', 'Fabfile.py']
     options = filter(os.path.exists, guesses)
     if options:
         return options[0]
@@ -915,8 +1014,8 @@ def _load_default_settings():
     if os.path.exists(cfg):
         comments = lambda s: s and not s.startswith("#")
         settings = filter(comments, open(cfg, 'r'))
-        settings = [(k.strip(),v.strip()) for k,_,v in
-            [s.partition('=') for s in settings]]
+        settings = [(k.strip(), v.strip()) for k, _, v in
+            [partition(s, '=') for s in settings]]
         ENV.update(settings)
 
 def _parse_args(args):
@@ -926,7 +1025,7 @@ def _parse_args(args):
         if ':' in cmd:
             cmd, cmd_str_args = cmd.split(':', 1)
             for cmd_arg_kv in cmd_str_args.split(','):
-                k, _, v = cmd_arg_kv.partition('=')
+                k, _, v = partition(cmd_arg_kv, '=')
                 cmd_args[k] = (v % ENV)
         cmds.append((cmd, cmd_args))
     return cmds
@@ -939,7 +1038,7 @@ def _validate_commands(cmds):
         for cmd in cmds:
             if not cmd[0] in COMMANDS:
                 print("No such command: %s" % cmd[0])
-                exit(1)
+                sys.exit(1)
 
 def _execute_commands(cmds):
     for cmd, args in cmds:
@@ -960,5 +1059,4 @@ def main(args):
         _execute_commands(commands)
     finally:
         _disconnect()
-        print("Done.")
 
